require('dotenv').config();
const express = require('express');
const path = require('path');
// const favicon = require('serve-favicon');
const logger = require('morgan');
const cookieParser = require('cookie-parser');
const bodyParser = require('body-parser');
const passport = require('passport');
const session = require('express-session');

const index = require('./routes/index');
const users = require('./routes/users');

const app = express();
// Passport-määritykset ovat erillisessä tiedostossa passportfunction.js
const passportFunction = require('./passportfunction');

// Otetaan yhteys passportdb-tietokantaan
require('./dbconnection');

// Suoritetaan passport-funktio aina kun sovellus käynnistyy
// Funktion argumenttina on yllä haettu passport-kirjasto
passportFunction(passport);

app.set('views', path.join(__dirname, 'views'));
app.set('view engine', 'ejs');

// uncomment after placing your favicon in /public
//app.use(favicon(path.join(__dirname, 'public', 'favicon.ico')));
app.use(logger('dev'));
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({ extended: false }));
app.use(cookieParser());
app.use(express.static(path.join(__dirname, 'public')));

app.use(session({ secret: process.env.SESSION_SECRET, resave: true, saveUninitialized: true })); // sovelluksen sessio käyttöön
app.use(passport.initialize());
app.use(passport.session()); // passportin login-sessio

//reitit
app.use('/', index);
app.use('/users', users);

// catch 404 and forward to error handler
app.use(function (req, res, next) {
    const err = new Error('Not Found');
    err.status = 404;
    next(err);
});

// error handler
app.use(function (err, req, res, next) {
    // set locals, only providing error in development
    res.locals.message = err.message;
    res.locals.error = req.app.get('env') === 'development' ? err : {};

    // render the error page
    res.status(err.status || 500);
    res.render('error');
});

module.exports = app;
