require('dotenv').config();
const GoogleStrategy = require('passport-google-oauth20').Strategy;
const User = require('./models/User');

/* Anonyymi funktio joka palauttaa Google-käyttäjän kannasta tai luo uuden käyttäjän kantaan
ja palauttaa sen. Käyttäjän tiedot saadaan Googlen login-applikaatiosta.
Funktio käyttää passport-kirjastoa joten passport-kirjasto tulee funktioon parametrina.
Tämä anonyymi funktio on importattu app.js -tiedostoon jossa se saa parametrikseen passportin.
Funktio ajetaan siis aina kun app.js käynnistyy.
 */
module.exports = function (passport) {

    // passportin login-session vaatii käyttäjän serialisointa (olio merkkijonoksi)
    // passportissa on tähän omat metodit
    // serialisointi
    passport.serializeUser(function (user, done) {
        done(null, user);
    });
    // ja deserialisointi
    passport.deserializeUser(function (id, done) {
        User.findById(id, function (err, user) {
            done(err, user);
        });
    });

    passport.use(new GoogleStrategy({
        // haetaan Googlen login-applikaation data ympäristömuuttujista
        clientID: process.env.GOOGLE_CLIENT_ID,
        clientSecret: process.env.GOOGLE_CLIENT_SECRET,
        callbackURL: process.env.CALLBACKURL,

    },
    function (token, refreshToken, profile, done) {
        // User.findOne ei käynnisty ennen kuin kaikki data on saatu Googlelta
        process.nextTick(function () {

            // Haetaan käyttäjää kannasta Googlelta saadun profile.id:n perusteella
            User.findOne({
            // katsotaan vastaako saatu profile.id jotakin kannassa olevaa google.id:tä
                'google.id': profile.id,
            }, function (err, user) {
                if (err) {
                    return done(err);
                }
                if (user) {
                    // jos käyttäjä löytyy kannasta se palautetaan
                    return done(null, user);
                } else {
                    // jos käyttäjää ei ole kannassa tehdään uusi käyttäjä-olio,
                    // joka tallennetaan kantaan
                    const newUser = new User();

                    newUser.google.id = profile.id;
                    newUser.google.token = token;
                    newUser.google.name = profile.displayName;
                    newUser.google.email = profile.emails[0].value;

                    // käyttäjä kantaan
                    newUser.save(function (err) {
                        if (err) {
                            throw err;
                        }
                        // palautetaan uusi käyttäjä
                        return done(null, newUser);
                    });
                }
            });
        });
    }));
};
